# Generated by Django 3.2.11 on 2022-03-16 18:47

import uuid

import _socket
import django.contrib.sites.managers
import django.core.validators
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_crypto_fields.fields.encrypted_text_field
import django_revision.revision_field
import edc_model.validators.date
import edc_model_fields.fields.other_charfield
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_action_item", "0028_auto_20210203_0706"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("sites", "0002_alter_domain_unique"),
        ("meta_prn", "0021_auto_20220316_2147"),
        ("meta_subject", "0100_auto_20220309_2238"),
    ]

    operations = [
        migrations.CreateModel(
            name="Delivery",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("tracking_identifier", models.CharField(max_length=32, unique=True)),
                ("action_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "informant_is_patient",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=5,
                        verbose_name="Was this information from study participant?",
                    ),
                ),
                (
                    "informant_contact",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=71,
                        null=True,
                        verbose_name="If information not from study participant, please give the name and contact details of the informant.",
                    ),
                ),
                (
                    "informant_relation",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("husband_wife", "Husband/wife"),
                            ("Parent", "Parent"),
                            ("child", "Child"),
                            ("unknown", "Unknown"),
                            ("OTHER", "Other"),
                        ],
                        max_length=25,
                        null=True,
                        verbose_name="Informants relationship to the participant?",
                    ),
                ),
                (
                    "informant_relation_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "delivery_datetime",
                    models.DateTimeField(
                        help_text="If TIME unknown, estimate",
                        validators=[
                            edc_model.validators.date.datetime_not_future,
                            edc_protocol.validators.datetime_not_before_study_start,
                        ],
                        verbose_name="Date and time of delivery :",
                    ),
                ),
                (
                    "delivery_time_estimated",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=3,
                        verbose_name="Is the delivery TIME estimated?",
                    ),
                ),
                (
                    "delivery_location",
                    models.CharField(
                        choices=[
                            ("home", "At home"),
                            ("hospital_clinic", "Hospital / Clinic"),
                            ("OTHER", "Other location, specify"),
                        ],
                        max_length=25,
                        verbose_name="Where did the delivery occur?",
                    ),
                ),
                (
                    "delivery_location_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "delivery_location_name",
                    models.CharField(
                        blank=True,
                        max_length=150,
                        null=True,
                        verbose_name="If delivery occurred at a `hospital` or `clinic`, please give name of the facility",
                    ),
                ),
                (
                    "delivery_ga",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(1),
                            django.core.validators.MaxValueValidator(45),
                        ],
                        verbose_name="Gestational age at delivery",
                    ),
                ),
                (
                    "maternal_outcome",
                    models.CharField(
                        choices=[
                            ("no_complications", "No complications"),
                            (
                                "complications_full_recovery",
                                "Complications with full recovery",
                            ),
                            (
                                "complications_ongoing_recovery",
                                "Complications with ongoing recovery",
                            ),
                            ("dead", "Maternal mortality"),
                        ],
                        max_length=50,
                        verbose_name="What was the maternal outcome of the pregnancy?",
                    ),
                ),
                (
                    "gm_treated",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=5,
                        verbose_name="Was the participant treated for gestational diabetes?",
                    ),
                ),
                (
                    "fetal_outcome_count",
                    models.IntegerField(
                        help_text="Each to be reported individually below",
                        verbose_name="Number of births / fetal / neonatal outcomes",
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Delivery",
                "verbose_name_plural": "Delivery",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "unique_together": {("subject_visit", "delivery_datetime")},
            },
            managers=[
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.RenameField(
            model_name="glucose",
            old_name="ifg_datetime",
            new_name="fbg_datetime",
        ),
        migrations.RenameField(
            model_name="glucose",
            old_name="ifg_not_performed_reason",
            new_name="fbg_not_performed_reason",
        ),
        migrations.RenameField(
            model_name="glucose",
            old_name="ifg_performed",
            new_name="fbg_performed",
        ),
        migrations.RenameField(
            model_name="glucose",
            old_name="ifg_quantifier",
            new_name="fbg_quantifier",
        ),
        migrations.RenameField(
            model_name="glucose",
            old_name="ifg_units",
            new_name="fbg_units",
        ),
        migrations.RenameField(
            model_name="glucose",
            old_name="ifg_value",
            new_name="fbg_value",
        ),
        migrations.RenameField(
            model_name="historicalglucose",
            old_name="ifg_datetime",
            new_name="fbg_datetime",
        ),
        migrations.RenameField(
            model_name="historicalglucose",
            old_name="ifg_not_performed_reason",
            new_name="fbg_not_performed_reason",
        ),
        migrations.RenameField(
            model_name="historicalglucose",
            old_name="ifg_performed",
            new_name="fbg_performed",
        ),
        migrations.RenameField(
            model_name="historicalglucose",
            old_name="ifg_quantifier",
            new_name="fbg_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalglucose",
            old_name="ifg_units",
            new_name="fbg_units",
        ),
        migrations.RenameField(
            model_name="historicalglucose",
            old_name="ifg_value",
            new_name="fbg_value",
        ),
        migrations.CreateModel(
            name="PregnancyUpdate",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default="COMPLETE",
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        help_text="for example, why some data is still pending",
                        null=True,
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "contact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="If yes, please specify below",
                        max_length=15,
                        verbose_name="Has the clinic received any updates on this pregnancy?",
                    ),
                ),
                (
                    "comment",
                    models.TextField(null=True, verbose_name="Comment / Updates"),
                ),
                (
                    "pregnancy_notification",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_prn.pregnancynotification",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Pregnancy Update",
                "verbose_name_plural": "Pregnancy Update",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
            managers=[
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalPregnancyUpdate",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default="COMPLETE",
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        help_text="for example, why some data is still pending",
                        null=True,
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "contact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="If yes, please specify below",
                        max_length=15,
                        verbose_name="Has the clinic received any updates on this pregnancy?",
                    ),
                ),
                (
                    "comment",
                    models.TextField(null=True, verbose_name="Comment / Updates"),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "pregnancy_notification",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_prn.pregnancynotification",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Pregnancy Update",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalDelivery",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("tracking_identifier", models.CharField(db_index=True, max_length=32)),
                ("action_identifier", models.CharField(db_index=True, max_length=50)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "informant_is_patient",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=5,
                        verbose_name="Was this information from study participant?",
                    ),
                ),
                (
                    "informant_contact",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=71,
                        null=True,
                        verbose_name="If information not from study participant, please give the name and contact details of the informant.",
                    ),
                ),
                (
                    "informant_relation",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("husband_wife", "Husband/wife"),
                            ("Parent", "Parent"),
                            ("child", "Child"),
                            ("unknown", "Unknown"),
                            ("OTHER", "Other"),
                        ],
                        max_length=25,
                        null=True,
                        verbose_name="Informants relationship to the participant?",
                    ),
                ),
                (
                    "informant_relation_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "delivery_datetime",
                    models.DateTimeField(
                        help_text="If TIME unknown, estimate",
                        validators=[
                            edc_model.validators.date.datetime_not_future,
                            edc_protocol.validators.datetime_not_before_study_start,
                        ],
                        verbose_name="Date and time of delivery :",
                    ),
                ),
                (
                    "delivery_time_estimated",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=3,
                        verbose_name="Is the delivery TIME estimated?",
                    ),
                ),
                (
                    "delivery_location",
                    models.CharField(
                        choices=[
                            ("home", "At home"),
                            ("hospital_clinic", "Hospital / Clinic"),
                            ("OTHER", "Other location, specify"),
                        ],
                        max_length=25,
                        verbose_name="Where did the delivery occur?",
                    ),
                ),
                (
                    "delivery_location_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "delivery_location_name",
                    models.CharField(
                        blank=True,
                        max_length=150,
                        null=True,
                        verbose_name="If delivery occurred at a `hospital` or `clinic`, please give name of the facility",
                    ),
                ),
                (
                    "delivery_ga",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(1),
                            django.core.validators.MaxValueValidator(45),
                        ],
                        verbose_name="Gestational age at delivery",
                    ),
                ),
                (
                    "maternal_outcome",
                    models.CharField(
                        choices=[
                            ("no_complications", "No complications"),
                            (
                                "complications_full_recovery",
                                "Complications with full recovery",
                            ),
                            (
                                "complications_ongoing_recovery",
                                "Complications with ongoing recovery",
                            ),
                            ("dead", "Maternal mortality"),
                        ],
                        max_length=50,
                        verbose_name="What was the maternal outcome of the pregnancy?",
                    ),
                ),
                (
                    "gm_treated",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=5,
                        verbose_name="Was the participant treated for gestational diabetes?",
                    ),
                ),
                (
                    "fetal_outcome_count",
                    models.IntegerField(
                        help_text="Each to be reported individually below",
                        verbose_name="Number of births / fetal / neonatal outcomes",
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Delivery",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalBirthOutcomes",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("tracking_identifier", models.CharField(db_index=True, max_length=32)),
                ("action_identifier", models.CharField(db_index=True, max_length=50)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                ("birth_order", models.IntegerField()),
                (
                    "birth_outcome",
                    models.CharField(
                        choices=[
                            ("live_at_term", "Live birth at term"),
                            ("live_preterm", "Live birth preterm"),
                            ("stillbirth", "Stillbirth"),
                            ("miscarriage", "Miscarriage"),
                        ],
                        max_length=25,
                        verbose_name="Outcome",
                    ),
                ),
                (
                    "birth_weight",
                    models.IntegerField(
                        blank=True,
                        help_text="gm",
                        null=True,
                        verbose_name="Weight (gm)",
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "delivery",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_subject.delivery",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Birth Outcomes",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="BirthOutcomes",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("tracking_identifier", models.CharField(max_length=32, unique=True)),
                ("action_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                ("birth_order", models.IntegerField()),
                (
                    "birth_outcome",
                    models.CharField(
                        choices=[
                            ("live_at_term", "Live birth at term"),
                            ("live_preterm", "Live birth preterm"),
                            ("stillbirth", "Stillbirth"),
                            ("miscarriage", "Miscarriage"),
                        ],
                        max_length=25,
                        verbose_name="Outcome",
                    ),
                ),
                (
                    "birth_weight",
                    models.IntegerField(
                        blank=True,
                        help_text="gm",
                        null=True,
                        verbose_name="Weight (gm)",
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "delivery",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_subject.delivery",
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Birth Outcomes",
                "verbose_name_plural": "Birth Outcomes",
                "ordering": ["delivery", "birth_order"],
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "unique_together": {("delivery", "birth_order")},
            },
            managers=[
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
    ]



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin

from ...form_validators import DmEndpointFormValidator
from ...models import DmEndpoint


class DmEndpointForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = DmEndpointFormValidator

    class Meta:
        model = DmEndpoint
        fields = "__all__"



# Generated by Django 3.0.4 on 2020-04-20 12:24

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0025_auto_20200420_1455"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="coronakap",
            options={
                "verbose_name": "Coronavirus Knowledge, Attitudes, and Practices",
                "verbose_name_plural": "Coronavirus Knowledge, Attitudes, and Practices",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalcoronakap",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Coronavirus Knowledge, Attitudes, and Practices",
            },
        ),
        migrations.AddIndex(
            model_name="coronakap",
            index=models.Index(
                fields=["subject_visit", "site", "id"],
                name="meta_subjec_subject_4bc9e9_idx",
            ),
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-15 23:32

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0193_alter_glucosefbg_fbg_value_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="glucosefbg",
            old_name="assay_datetime",
            new_name="fbg_datetime",
        ),
        migrations.RenameField(
            model_name="historicalglucosefbg",
            old_name="assay_datetime",
            new_name="fbg_datetime",
        ),
        migrations.AddField(
            model_name="glucosefbg",
            name="fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="glucosefbg",
            name="fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Expected something like 1h20m, 11h5m, etc. No spaces allowed.",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AddField(
            model_name="historicalglucosefbg",
            name="fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalglucosefbg",
            name="fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Expected something like 1h20m, 11h5m, etc. No spaces allowed.",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AlterField(
            model_name="glucosefbg",
            name="fasting",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="As reported by patient",
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AlterField(
            model_name="glucosefbg",
            name="fbg_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
                verbose_name="FBG quantifier",
            ),
        ),
        migrations.AlterField(
            model_name="glucosefbg",
            name="fbg_units",
            field=models.CharField(
                choices=[
                    ("mg/dL", "mg/dL"),
                    ("mmol/L", "mmol/L (millimoles/L)"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="glucosefbg",
            name="fbg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucosefbg",
            name="fasting",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="As reported by patient",
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucosefbg",
            name="fbg_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), (">=", ">="), ("<", "<"), ("<=", "<=")],
                default="=",
                max_length=10,
                verbose_name="FBG quantifier",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucosefbg",
            name="fbg_units",
            field=models.CharField(
                choices=[
                    ("mg/dL", "mg/dL"),
                    ("mmol/L", "mmol/L (millimoles/L)"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucosefbg",
            name="fbg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
    ]



# Generated by Django 4.2.5 on 2023-10-17 22:25

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_lists", "0014_auto_20220913_2139"),
    ]

    operations = [
        migrations.AddField(
            model_name="abnormalfootappearanceobservations",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="arvregimens",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="baselinesymptoms",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="diabetessymptoms",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="hypertensionmedications",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="nonadherencereasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="offstudyreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="oiprophylaxis",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="subjectvisitmissedreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="symptoms",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="transferreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
    ]



from datetime import datetime
from zoneinfo import ZoneInfo

import time_machine
from dateutil.relativedelta import relativedelta
from django.test import TestCase
from edc_constants.constants import NO, YES
from edc_utils import get_utcnow
from model_bakery import baker

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_visit_schedule.constants import MONTH1, MONTH3, MONTH6, MONTH9, MONTH12, WEEK2


@time_machine.travel(datetime(2022, 6, 11, 8, 00, tzinfo=ZoneInfo("UTC")))
class TestMnsiRequired(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.baseline_datetime = get_utcnow() - relativedelta(months=6)

        six_months_ago = self.baseline_datetime
        self.subject_screening = self.get_subject_screening(
            report_datetime=six_months_ago,
            eligibility_datetime=six_months_ago,
        )
        self.subject_consent = self.get_subject_consent(
            self.subject_screening,
            consent_datetime=six_months_ago,
        )
        self.subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            appt_datetime=six_months_ago,
        )

    def get_week2_visit(self):
        return self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=WEEK2,
            appt_datetime=self.baseline_datetime + relativedelta(weeks=2),
        )

    def get_month1_visit(self):
        return self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=MONTH1,
            appt_datetime=self.baseline_datetime + relativedelta(months=1),
        )

    def get_month3_visit(self):
        return self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=MONTH3,
            appt_datetime=self.baseline_datetime + relativedelta(months=3),
        )

    def get_month6_visit(self):
        return self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=MONTH6,
            appt_datetime=self.baseline_datetime + relativedelta(months=6),
        )

    def get_month9_visit(self):
        return self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=MONTH9,
            appt_datetime=self.baseline_datetime + relativedelta(months=9),
        )

    @staticmethod
    def set_mnsi_status(subject_visit, mnsi_performed):
        visit_mnsi = baker.make(
            "meta_subject.mnsi",
            subject_visit=subject_visit,
            mnsi_performed=mnsi_performed,
        )
        visit_mnsi.save()

    def test_mnsi_not_required_at_2w(self):
        crfs = self.get_crf_metadata(self.get_week2_visit())
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_required_at_1m(self):
        self.get_week2_visit()

        month1_visit = self.get_month1_visit()
        crfs = self.get_crf_metadata(month1_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_3m_if_already_performed_at_1m(self):
        self.get_week2_visit()

        # MNSI shouldn't be required at any point after it has been performed
        month1_visit = self.get_month1_visit()
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=YES)

        month3_visit = self.get_month3_visit()
        crfs = self.get_crf_metadata(month3_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_required_at_3m_if_not_performed_at_1m(self):
        self.get_week2_visit()

        month1_visit = self.get_month1_visit()
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        month3_visit = self.get_month3_visit()
        crfs = self.get_crf_metadata(month3_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_6m_if_already_performed_at_1m(self):
        self.get_week2_visit()

        # MNSI shouldn't be required at any point after it has been performed
        month1_visit = self.get_month1_visit()
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=YES)

        month3_visit = self.get_month3_visit()
        crfs = self.get_crf_metadata(month3_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

        month6_visit = self.get_month6_visit()
        crfs = self.get_crf_metadata(month6_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_6m_if_already_performed_at_3m(self):
        self.get_week2_visit()

        month1_visit = self.get_month1_visit()
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        # MNSI shouldn't be required at any point after it has been performed
        month3_visit = self.get_month3_visit()
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=YES)

        month6_visit = self.get_month6_visit()
        crfs = self.get_crf_metadata(month6_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_required_at_6m_if_not_performed_by_3m(self):
        self.get_week2_visit()

        month1_visit = self.get_month1_visit()
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        month3_visit = self.get_month3_visit()
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=NO)

        month6_visit = self.get_month6_visit()
        crfs = self.get_crf_metadata(month6_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_9m_if_already_performed_at_1m(self):
        self.get_week2_visit()

        # MNSI shouldn't be required at any point after it has been performed
        month1_visit = self.get_month1_visit()
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=YES)

        month3_visit = self.get_month3_visit()
        crfs = self.get_crf_metadata(month3_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

        month6_visit = self.get_month6_visit()
        crfs = self.get_crf_metadata(month6_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

        month9_visit = self.get_month9_visit()
        crfs = self.get_crf_metadata(month9_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_9m_if_already_performed_at_3m(self):
        self.get_week2_visit()

        month1_visit = self.get_month1_visit()
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        # MNSI shouldn't be required at any point after it has been performed
        month3_visit = self.get_month3_visit()
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=YES)

        month6_visit = self.get_month6_visit()
        crfs = self.get_crf_metadata(month6_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

        month9_visit = self.get_month9_visit()
        crfs = self.get_crf_metadata(month9_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_9m_if_already_performed_at_6m(self):
        self.get_week2_visit()

        month1_visit = self.get_month1_visit()
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        month3_visit = self.get_month3_visit()
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=NO)

        # MNSI shouldn't be required at any point after it has been performed
        month6_visit = self.get_month6_visit()
        self.set_mnsi_status(subject_visit=month6_visit, mnsi_performed=YES)

        month9_visit = self.get_month9_visit()
        crfs = self.get_crf_metadata(month9_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_9m_even_if_not_performed_by_6m(self):
        self.get_week2_visit()

        month1_visit = self.get_month1_visit()
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        month3_visit = self.get_month3_visit()
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=NO)

        month6_visit = self.get_month6_visit()
        self.set_mnsi_status(subject_visit=month6_visit, mnsi_performed=NO)

        month9_visit = self.get_month9_visit()
        crfs = self.get_crf_metadata(month9_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_12m_even_if_not_performed_by_6m(self):
        self.get_week2_visit()

        month1_visit = self.get_month1_visit()
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        month3_visit = self.get_month3_visit()
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=NO)

        month6_visit = self.get_month6_visit()
        self.set_mnsi_status(subject_visit=month6_visit, mnsi_performed=NO)

        self.get_month9_visit()

        month12_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=MONTH12,
            appt_datetime=self.baseline_datetime + relativedelta(months=12),
        )
        crfs = self.get_crf_metadata(month12_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])



# Generated by Django 5.0.8 on 2024-08-22 21:12

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0032_alter_endpointsproxy_options"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                '# noqa\nselect *, uuid() as id, now() as created, \'meta_reports.glucose_summary_view\' as report_model\nfrom (\n    select v.subject_identifier, fbg_value, fbg_datetime, null as \'ogtt_value\', null as \'ogtt_datetime\',\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as \'fasted\',\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucosefbg as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier\n    UNION\n    select v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as \'fasted\',\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucose as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier\n) as A\norder by subject_identifier, fbg_datetime',
                "glucose_summary_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                '# noqa\nselect *, uuid() as id, now() as created, \'meta_reports.glucose_summary_view\' as report_model\nfrom (\n    select subject_identifier, fbg_value, fbg_datetime, null as \'ogtt_value\', null as \'ogtt_datetime\',\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as \'fasted\',\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id\n    from meta_subject_glucosefbg as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    UNION\n    select subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as \'fasted\',\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id\n    from meta_subject_glucose as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id) as A\norder by subject_identifier, fbg_datetime',
                "glucose_summary_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                '# noqa\nselect *, get_random_uuid() as id, now() as created, \'meta_reports.glucose_summary_view\' as report_model\nfrom (\n    select v.subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucosefbg as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier\n    UNION\n    select v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucose as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier\n) as A\norder by subject_identifier, fbg_datetime',
                "glucose_summary_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                '# noqa\nselect *, get_random_uuid() as id, now() as created, \'meta_reports.glucose_summary_view\' as report_model\nfrom (\n    select subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id\n    from meta_subject_glucosefbg as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    UNION\n    select subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id\n    from meta_subject_glucose as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id) as A\norder by subject_identifier, fbg_datetime',
                "glucose_summary_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                '# noqa\nSELECT *, lower(\n    hex(randomblob(4)) || \'-\' || hex(randomblob(2)) || \'-\' || \'4\' ||\n    substr(hex( randomblob(2)), 2) || \'-\' ||\n    substr(\'AB89\', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || \'-\' ||\n    hex(randomblob(6))\n  ) as id, datetime() as `created`, \'meta_reports.glucose_summary_view\' as report_model\nfrom (\n    select v.subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucosefbg as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier\n    UNION\n    select v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucose as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier    \n    ) as A\norder by subject_identifier, fbg_datetime',
                "glucose_summary_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                '# noqa\nSELECT *, lower(\n    hex(randomblob(4)) || \'-\' || hex(randomblob(2)) || \'-\' || \'4\' ||\n    substr(hex( randomblob(2)), 2) || \'-\' ||\n    substr(\'AB89\', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || \'-\' ||\n    hex(randomblob(6))\n  ) as id, datetime() as `created`, \'meta_reports.glucose_summary_view\' as report_model\nfrom (\n    select subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id\n    from meta_subject_glucosefbg as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    UNION\n    select subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id\n    from meta_subject_glucose as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id) as A\norder by subject_identifier, fbg_datetime',
                "glucose_summary_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
    ]



# Generated by Django 5.0 on 2024-01-09 23:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_rando", "0004_alter_randomizationlist_options_and_more"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="randomizationlist",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "permissions": (("display_assignment", "Can display assignment"),),
                "verbose_name": "Randomization List (Phase Three)",
                "verbose_name_plural": "Randomization List (Phase Three)",
            },
        ),
        migrations.AlterUniqueTogether(
            name="randomizationlist",
            unique_together=set(),
        ),
        migrations.AddField(
            model_name="historicalrandomizationlist",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalrandomizationlist",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="randomizationlist",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="randomizationlist",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddIndex(
            model_name="randomizationlist",
            index=models.Index(
                fields=["modified", "created"], name="meta_rando__modifie_135549_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="randomizationlist",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="meta_rando__user_mo_c334b9_idx",
            ),
        ),
        migrations.AddConstraint(
            model_name="randomizationlist",
            constraint=models.UniqueConstraint(
                fields=("site_name", "sid"),
                name="meta_rando_randomizationlist_site_sid_uniq",
            ),
        ),
    ]



# Generated by Django 2.2.7 on 2019-12-13 00:14

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0016_auto_20191119_2331"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="weight",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="in kgs",
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(15),
                    django.core.validators.MaxValueValidator(180),
                ],
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="weight",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="in kgs",
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(15),
                    django.core.validators.MaxValueValidator(180),
                ],
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="weight",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="in kgs",
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(15),
                    django.core.validators.MaxValueValidator(180),
                ],
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="weight",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="in kgs",
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(15),
                    django.core.validators.MaxValueValidator(180),
                ],
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="weight",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="in kgs",
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(15),
                    django.core.validators.MaxValueValidator(180),
                ],
            ),
        ),
    ]



from edc_qareports.sql_generator import SqlViewGenerator

from .qa_cases import qa_cases


def get_view_definition() -> dict:
    subquery = " UNION ".join([qa_case.sql for qa_case in qa_cases])
    sql_view = SqlViewGenerator(
        report_model="onstudy_missing_values_view",
        ordering=["subject_identifier", "site_id"],
    )

    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



# Generated by Django 3.2.11 on 2022-03-16 18:47

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0039_auto_20220312_1938"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Repeat FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Repeat OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_ogtt_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fbg_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="inclusion_a",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI>30 combined with FBG (6.1 to 6.9 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="inclusion_b",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI>30 combined with OGTT (2 hours) (7.0 to 11.10 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="inclusion_c",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI<=30 combined with FBG (6.3 to 6.9 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="inclusion_d",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI<=30 combined with OGTT (2 hours) (9.0 to 11.10 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Repeat OGTT Units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ogtt_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="OGTT Units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Repeat FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Repeat OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_ogtt_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fbg_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="inclusion_a",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI>30 combined with FBG (6.1 to 6.9 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="inclusion_b",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI>30 combined with OGTT (2 hours) (7.0 to 11.10 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="inclusion_c",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI<=30 combined with FBG (6.3 to 6.9 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="inclusion_d",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI<=30 combined with OGTT (2 hours) (9.0 to 11.10 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Repeat OGTT Units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ogtt_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="OGTT Units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Repeat FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Repeat OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_ogtt_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fbg_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="inclusion_a",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI>30 combined with FBG (6.1 to 6.9 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="inclusion_b",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI>30 combined with OGTT (2 hours) (7.0 to 11.10 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="inclusion_c",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI<=30 combined with FBG (6.3 to 6.9 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="inclusion_d",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI<=30 combined with OGTT (2 hours) (9.0 to 11.10 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Repeat OGTT Units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ogtt_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="OGTT Units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Repeat FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Repeat OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_ogtt_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fbg_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="inclusion_a",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI>30 combined with FBG (6.1 to 6.9 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="inclusion_b",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI>30 combined with OGTT (2 hours) (7.0 to 11.10 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="inclusion_c",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI<=30 combined with FBG (6.3 to 6.9 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="inclusion_d",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI<=30 combined with OGTT (2 hours) (9.0 to 11.10 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Repeat OGTT Units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ogtt_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="OGTT Units",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Repeat FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_ogtt2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="Repeat OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_ogtt_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="OGTT (2-hours)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fbg_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="inclusion_a",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI>30 combined with FBG (6.1 to 6.9 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="inclusion_b",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI>30 combined with OGTT (2 hours) (7.0 to 11.10 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="inclusion_c",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI<=30 combined with FBG (6.3 to 6.9 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="inclusion_d",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                default="tbd",
                help_text="system calculated",
                max_length=15,
                verbose_name="BMI<=30 combined with OGTT (2 hours) (9.0 to 11.10 mmol/L)",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Repeat OGTT Units",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ogtt_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="OGTT Units",
            ),
        ),
    ]



from edc_constants.constants import OTHER, YES
from edc_qareports.sql_generator import CrfCase

qa_cases = [
    CrfCase(
        label="No HIV Diagnosis date",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        fld_name="hiv_diagnosis_date",
    ),
    CrfCase(
        label="No VL value or VL date",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        fld_name="viral_load_date",
        where="crf.viral_load is null or crf.viral_load_date is null",
    ),
    CrfCase(
        label="No CD4 value or CD4 date",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        where="crf.cd4 is null or crf.cd4_date is null",
    ),
    CrfCase(
        label="No current ARV start date",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        fld_name="current_arv_regimen_start_date",
    ),
    CrfCase(
        label="No current ARV start date but previous ARV is YES",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        where=(
            f"crf.has_previous_arv_regimen ='{YES}' and "
            "crf.current_arv_regimen_start_date is null"
        ),
    ),
    CrfCase(
        label="Other current ARV regimen missing",
        dbtable="meta_subject_patienthistory",
        label_lower="meta_subject.patienthistory",
        where=f"crf.other_current_arv_regimen is null and arvregimen.name='{OTHER}'",
        list_tables=[("current_arv_regimen_id", "meta_lists_arvregimens", "arvregimen")],
    ),
    CrfCase(
        label="FBG/OGTT: missing OGTT",
        dbtable="meta_subject_glucose",
        label_lower="meta_subject.glucose",
        where=(
            "(crf.fbg_value is not null and crf.ogtt_value is null) "
            f"and ogtt_performed='{YES}'"
        ),
    ),
]



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from edc_visit_schedule.constants import OFFSCHEDULE_ACTION

from .constants import OFFSCHEDULE_PREGNANCY_ACTION

if TYPE_CHECKING:
    from .models import OffSchedulePregnancy


class PregnancyActionItemMixin:
    """A mixin for off schedule action items"""

    def get_next_offschedule_action(self) -> str:
        """Returns next off schedule action name.

        Called by get_next_actions().
        """
        try:
            self.offschedule_pregnancy_model_cls.objects.get(
                subject_identifier=self.subject_identifier
            )
        except ObjectDoesNotExist:
            return OFFSCHEDULE_ACTION
        else:
            return OFFSCHEDULE_PREGNANCY_ACTION

    @property
    def offschedule_pregnancy_model_cls(self) -> OffSchedulePregnancy:
        return django_apps.get_model("meta_prn.offschedulepregnancy")



from dateutil.relativedelta import relativedelta
from django.test import TestCase, tag
from edc_appointment.models import Appointment
from edc_constants.constants import FEMALE, MALE, YES
from edc_utils import get_utcnow
from edc_visit_schedule.constants import MONTH1
from model_bakery.baker import make_recipe

from meta_consent.models import SubjectConsent
from meta_screening.models import SubjectScreening
from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_subject.forms import FollowupExaminationForm
from meta_visit_schedule.constants import DELIVERY, MONTH3, SCHEDULE_PREGNANCY


class TestMetadataRules(MetaTestCaseMixin, TestCase):
    def test_ok(self):
        self.subject_visit = self.get_subject_visit(gender=FEMALE)
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        obj = make_recipe("meta_subject.followupexamination", subject_visit=subject_visit)
        form = FollowupExaminationForm(instance=obj)
        form.is_valid()

    def test_pregnancy_not_required_for_male(self):
        self.subject_visit = self.get_subject_visit(gender=MALE)
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)

        self.assertNotIn(
            "meta_subject.urinepregnancy",
            [obj.model for obj in self.get_crf_metadata(subject_visit)],
        )

        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertNotIn(
            "meta_subject.urinepregnancy",
            [obj.model for obj in self.get_crf_metadata(subject_visit)],
        )

    def test_pregnancy_and_notification(self):
        self.subject_visit = self.get_subject_visit(gender=FEMALE)
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        urine_pregnancy = make_recipe(
            "meta_subject.urinepregnancy",
            subject_visit=subject_visit,
            report_datetime=get_utcnow(),
            assay_date=get_utcnow().date(),
        )
        self.assertFalse(urine_pregnancy.notified)
        self.assertIsNone(urine_pregnancy.notified_datetime)
        self.assertEqual(
            subject_visit.subject_identifier,
            urine_pregnancy.subject_visit.subject_identifier,
        )
        make_recipe(
            "meta_prn.pregnancynotification",
            subject_identifier=subject_visit.subject_identifier,
        )

    def test_only_pregnancy_update_required_after_pregnancy_notification(self):
        self.subject_visit = self.get_subject_visit(gender=FEMALE)
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        urine_pregnancy = make_recipe(
            "meta_subject.urinepregnancy",
            subject_visit=subject_visit,
            report_datetime=subject_visit.report_datetime,
            assay_date=subject_visit.report_datetime.date(),
        )
        self.assertFalse(urine_pregnancy.notified)
        self.assertIsNone(urine_pregnancy.notified_datetime)
        self.assertEqual(
            subject_visit.subject_identifier,
            urine_pregnancy.subject_visit.subject_identifier,
        )
        make_recipe(
            "meta_prn.pregnancynotification",
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=subject_visit.report_datetime,
            bhcg_confirmed=YES,
            unconfirmed_details="blah blah",
            edd=subject_visit.report_datetime + relativedelta(months=6),
        )

        Appointment.objects.get(
            subject_identifier=subject_visit.subject_identifier,
            schedule_name=SCHEDULE_PREGNANCY,
        )
        subject_screening = SubjectScreening.objects.get(
            subject_identifier=subject_visit.subject_identifier
        )
        subject_consent = SubjectConsent.objects.get(
            subject_identifier=subject_visit.subject_identifier
        )
        subject_visit = self.get_subject_visit(
            visit_code=DELIVERY,
            subject_screening=subject_screening,
            subject_consent=subject_consent,
        )
        self.assertEqual(
            ["meta_subject.delivery"],
            [obj.model for obj in self.get_crf_metadata(subject_visit)],
        )

    @tag("4")
    def test_health_economics_update_required(self):
        """Assert healtheconomicsupdate is required starting w/1030"""
        self.subject_visit = self.get_subject_visit(gender=FEMALE)
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH3)
        self.assertIn(
            "meta_subject.healtheconomicsupdate",
            [obj.model for obj in self.get_crf_metadata(subject_visit)],
        )

        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertIn(
            "meta_subject.healtheconomicsupdate",
            [obj.model for obj in self.get_crf_metadata(subject_visit)],
        )
        make_recipe(
            "meta_subject.healtheconomicsupdate",
            subject_visit=subject_visit,
            report_datetime=subject_visit.report_datetime,
        )

        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertNotIn(
            "meta_subject.healtheconomicsupdate",
            [obj.model for obj in self.get_crf_metadata(subject_visit)],
        )



# Generated by Django 3.0.6 on 2020-05-27 23:52
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0041_auto_20200528_0242"),
    ]

    operations = []



from django.test import TestCase

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin


class TestSubjectConsent(MetaTestCaseMixin, TestCase):
    def test_(self):
        subject_screening = self.get_subject_screening()
        subject_consent = self.get_subject_consent(subject_screening)
        self.assertIsNotNone(subject_consent.subject_identifier)
